#pragma once

#include <sstream>

#include <ext/ostream>
#include <ext/istream>

namespace ext {

class ostringstream : public ostream {
public:
	ostringstream ( ) : ostream ( new std::stringbuf ( ) ) {
	}

	ostringstream ( ostringstream && rhs ) noexcept : ostream ( std::move ( rhs ) ) {
		rhs = ostringstream ( );
	}

	~ostringstream ( ) noexcept {
		delete rdbuf ( );
	}

	ostringstream & operator = ( ostringstream && ) noexcept = default;

	void swap ( ostringstream & other ) {
		ostringstream tmp ( std::move ( * this ) );
		* this = std::move ( other );
		other = std::move ( tmp );
	}

	std::string str ( ) const & {
		return rdbuf ( )->str ( );
	}

	void str ( const std::string & str ) {
		return rdbuf ( )->str ( str );
	}

	std::stringbuf * rdbuf ( ) {
		return static_cast < std::stringbuf * > ( ostream::rdbuf ( ) );
	}

	const std::stringbuf * rdbuf ( ) const {
		return static_cast < const std::stringbuf * > ( ostream::rdbuf ( ) );
	}
};

class istringstream : public istream {
public:
	istringstream ( const std::string & data = "" ) : istream ( new std::stringbuf ( data ) ) {
	}

	istringstream ( istringstream && rhs ) noexcept : istream ( std::move ( rhs ) ) {
		rhs = istringstream ( );
	}

	~istringstream ( ) noexcept {
		delete rdbuf ( );
	}

	istringstream & operator = ( istringstream && ) noexcept = default;

	void swap ( istringstream & other ) {
		istringstream tmp ( std::move ( * this ) );
		* this = std::move ( other );
		other = std::move ( tmp );
	}

	std::string str ( ) const & {
		return rdbuf ( )->str ( );
	}

	void str ( const std::string & str ) {
		return rdbuf ( )->str ( str );
	}

	std::stringbuf * rdbuf ( ) {
		return static_cast < std::stringbuf * > ( istream::rdbuf ( ) );
	}

	const std::stringbuf * rdbuf ( ) const {
		return static_cast < const std::stringbuf * > ( istream::rdbuf ( ) );
	}
};

} /* namespace ext */
